@@ -7,44 +7,84 @@
 
 /* Py_ssize_t support was added in Python 2.5 */
 #if PY_VERSION_HEX < 0x02050000
-#ifndef PY_SSIZE_T_MAX /* patched Pyrex? */
-  typedef int Py_ssize_t;
-  #define PY_SSIZE_T_MAX INT_MAX
-  #define PY_SSIZE_T_MIN INT_MIN
-  #define PyInt_FromSsize_t(z) PyInt_FromLong(z)
-  #define PyInt_AsSsize_t(o)   PyInt_AsLong(o)
-#endif
+#  ifndef PY_SSIZE_T_MAX /* patched Pyrex? */
+     typedef int Py_ssize_t;
+#    define PY_SSIZE_T_MAX INT_MAX
+#    define PY_SSIZE_T_MIN INT_MIN
+#    define PyInt_FromSsize_t(z) PyInt_FromLong(z)
+#    define PyInt_AsSsize_t(o)   PyInt_AsLong(o)
+#  endif
 #endif
 
 /* Threading can crash under Python <= 2.4.1 */
 #if PY_VERSION_HEX < 0x02040200
-#ifndef WITHOUT_THREADING
-  #define WITHOUT_THREADING
-#endif
+#  ifndef WITHOUT_THREADING
+#    define WITHOUT_THREADING
+#  endif
 #endif
 
 #ifdef WITHOUT_THREADING
-  #define PyEval_SaveThread() (NULL)
-  #define PyEval_RestoreThread(state)
-  #define PyGILState_Ensure() (PyGILState_UNLOCKED)
-  #define PyGILState_Release(state)
+#  define PyEval_SaveThread() (NULL)
+#  define PyEval_RestoreThread(state)
+#  define PyGILState_Ensure() (PyGILState_UNLOCKED)
+#  define PyGILState_Release(state)
+#  undef  Py_UNBLOCK_THREADS
+#  define Py_UNBLOCK_THREADS
+#  undef  Py_BLOCK_THREADS
+#  define Py_BLOCK_THREADS
 #endif
 
 #ifdef WITHOUT_THREADING
-  #define ENABLE_THREADING 0
+#  define ENABLE_THREADING 0
 #else
-  #define ENABLE_THREADING 1
+#  define ENABLE_THREADING 1
 #endif
 
 /* libxml2 version specific setup */
 #include "libxml/xmlversion.h"
 #if LIBXML_VERSION < 20621
 /* (X|HT)ML_PARSE_COMPACT were added in libxml2 2.6.21 */
-#define XML_PARSE_COMPACT  0
-#define HTML_PARSE_COMPACT 0
+#  define XML_PARSE_COMPACT  0
+#  define HTML_PARSE_COMPACT 0
 
 /* HTML_PARSE_RECOVER was added in libxml2 2.6.21 */
-#define HTML_PARSE_RECOVER XML_PARSE_RECOVER
+#  define HTML_PARSE_RECOVER XML_PARSE_RECOVER
+#endif
+
+/* added to xmlsave API in libxml2 2.6.23 */
+#if LIBXML_VERSION < 20623
+#  define xmlSaveToBuffer(buffer, encoding, options)
+#endif
+
+/* added to xmlsave API in libxml2 2.6.22 */
+#if LIBXML_VERSION < 20622
+#  define XML_SAVE_NO_EMPTY   1<<2, /* no empty tags */
+#  define XML_SAVE_NO_XHTML   1<<3  /* disable XHTML1 specific rules */
+#endif
+
+/* added to xmlsave API in libxml2 2.6.21 */
+#if LIBXML_VERSION < 20621
+#  define XML_SAVE_NO_DECL    1<<1, /* drop the xml declaration */
+#endif
+
+/* schematron was added in libxml2 2.6.21 */
+#ifdef LIBXML_SCHEMATRON_ENABLED
+#  define ENABLE_SCHEMATRON 1
+#else
+#  define ENABLE_SCHEMATRON 0
+#  define XML_SCHEMATRON_OUT_QUIET 0
+#  define XML_SCHEMATRON_OUT_XML 0
+   typedef void xmlSchematron;
+   typedef void xmlSchematronParserCtxt;
+   typedef void xmlSchematronValidCtxt;
+#  define xmlSchematronNewDocParserCtxt(doc) NULL
+#  define xmlSchematronNewParserCtxt(file) NULL
+#  define xmlSchematronParse(ctxt) NULL
+#  define xmlSchematronFreeParserCtxt(ctxt)
+#  define xmlSchematronFree(schema)
+#  define xmlSchematronNewValidCtxt(schema, options) NULL
+#  define xmlSchematronValidateDoc(ctxt, doc) 0
+#  define xmlSchematronFreeValidCtxt(ctxt)
 #endif
 
 /* work around MSDEV 6.0 */
@@ -54,23 +94,44 @@ long _ftol2( double dblSource ) { return _ftol( dblSource ); }
 #endif
 
 /* Redefinition of some Python builtins as C functions */
-#define isinstance(o,c) PyObject_IsInstance(o,c)
-#define issubclass(c,csuper) PyObject_IsSubclass(c,csuper)
-#define hasattr(o,a)    PyObject_HasAttr(o,a)
-#define getattr(o,a)    PyObject_GetAttr(o,a)
 #define callable(o)     PyCallable_Check(o)
-#define str(o)          PyObject_Str(o)
-#define repr(o)         PyObject_Repr(o)
-#define iter(o)         PyObject_GetIter(o)
 #define _cstr(s)        PyString_AS_STRING(s)
+#define _fqtypename(o)  (((PyTypeObject*)o)->ob_type->tp_name)
+
+#ifdef __GNUC__
+/* Test for GCC > 2.95 */
+#if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)) 
+#define unlikely_condition(x) __builtin_expect((x), 0)
+#else /* __GNUC__ > 2 ... */
+#define unlikely_condition(x) (x)
+#endif /* __GNUC__ > 2 ... */
+#else /* __GNUC__ */
+#define unlikely_condition(x) (x)
+#endif /* __GNUC__ */
+
+static PyObject* __PY_NEW_GLOBAL_EMPTY_TUPLE = NULL;
+
+#define PY_NEW(T) \
+     (((PyTypeObject*)(T))->tp_new( \
+         (PyTypeObject*)(T), \
+         (unlikely_condition(__PY_NEW_GLOBAL_EMPTY_TUPLE == NULL) ? \
+             (__PY_NEW_GLOBAL_EMPTY_TUPLE = PyTuple_New(0)) : \
+             (__PY_NEW_GLOBAL_EMPTY_TUPLE)), \
+         NULL))
 
 #define _isString(obj)   PyObject_TypeCheck(obj, &PyBaseString_Type)
 
 #define _isElement(c_node) \
         (((c_node)->type == XML_ELEMENT_NODE) || \
-	 ((c_node)->type == XML_COMMENT_NODE) || \
+         ((c_node)->type == XML_COMMENT_NODE) || \
+         ((c_node)->type == XML_ENTITY_REF_NODE) || \
          ((c_node)->type == XML_PI_NODE))
 
+#define _isElementOrXInclude(c_node) \
+        (_isElement(c_node)                     || \
+         ((c_node)->type == XML_XINCLUDE_START) || \
+         ((c_node)->type == XML_XINCLUDE_END))
+
 #define _getNs(c_node) \
         (((c_node)->ns == 0) ? 0 : ((c_node)->ns->href))
 
