@@ -14,7 +14,10 @@ try:
 except ImportError:
     # DictMixin was introduced in Python 2.4
     from lxml.html._dictmixin import DictMixin
-import sets
+try:
+    set
+except NameError:
+    from sets import Set as set
 
 __all__ = [
     'document_fromstring', 'fragment_fromstring', 'fragments_fromstring', 'fromstring',
@@ -194,7 +197,7 @@ class HtmlMixin(object):
         that pre-compiling the expression can provide a substantial
         speedup.
         """
-        return cssselect.CSSSelect(expr)(self)
+        return cssselect.CSSSelector(expr)(self)
 
     ########################################
     ## Link functions
@@ -713,11 +716,11 @@ def submit_form(form, extra_values=None, open_http=None):
 
     You can use this like::
 
-        >>> form = doc.forms[0]                        # doctest: +SKIP
-        >>> form.inputs['foo'].value = 'bar' # etc     # doctest: +SKIP
-        >>> response = form.submit()                   # doctest: +SKIP
-        >>> doc = parse(response)                      # doctest: +SKIP
-        >>> doc.make_links_absolute(response.geturl()) # doctest: +SKIP
+        form = doc.forms[0]
+        form.inputs['foo'].value = 'bar' # etc
+        response = form.submit()
+        doc = parse(response)
+        doc.make_links_absolute(response.geturl())
 
     To change the HTTP requester, pass a function as ``open_http`` keyword
     argument that opens the URL for you.  The function must have the following
@@ -827,7 +830,7 @@ class InputGetter(object):
         return bool(results)
 
     def keys(self):
-        names = sets.Set()
+        names = set()
         for el in self:
             if el.name is not None:
                 names.add(el.name)
